package com.redhat;

import java.util.Collections;

import org.drools.KnowledgeBase;
import org.drools.KnowledgeBaseConfiguration;
import org.drools.KnowledgeBaseFactory;
import org.drools.builder.KnowledgeBuilder;
import org.drools.builder.KnowledgeBuilderFactory;
import org.drools.builder.ResourceType;
import org.drools.conf.MBeansOption;
import org.drools.io.ResourceFactory;
import org.drools.runtime.StatefulKnowledgeSession;

public class Main {
    private static final int MAXIMUM = 10000;
    private static final int SLEEP = 1000;

    public static void main(String[] args) {
        KnowledgeBuilder kbuilder = KnowledgeBuilderFactory.newKnowledgeBuilder();
        kbuilder.add(ResourceFactory.newClassPathResource("rules.drl", Main.class), ResourceType.DRL);
        kbuilder.add(ResourceFactory.newClassPathResource("process.bpmn2", Main.class), ResourceType.BPMN2);

        if (kbuilder.hasErrors()) throw new RuntimeException(kbuilder.getErrors().toString());

        KnowledgeBaseConfiguration kbconf = KnowledgeBaseFactory.newKnowledgeBaseConfiguration();
        kbconf.setOption(MBeansOption.ENABLED);

        KnowledgeBase kbase = KnowledgeBaseFactory.newKnowledgeBase(kbconf);
        kbase.addKnowledgePackages(kbuilder.getKnowledgePackages());

        StatefulKnowledgeSession ksession = kbase.newStatefulKnowledgeSession();

        ksession.setGlobal("maximum", MAXIMUM);

        for (int i = 0; i < MAXIMUM; i++) {
            long number = 2 + Math.round(Math.sin(i));
            for (int j = 0; j < number; j++) {
                ksession.insert(new String(Long.toString(number)));
            }
            ksession.insert(new NumberHolder(i));
            ksession.startProcess("com.redhat.process", Collections.<String, Object>singletonMap("round", i));

            ksession.fireAllRules();

            try { Thread.sleep(number * SLEEP); } catch (InterruptedException ex) {}
        }

        ksession.dispose();
    }

}
